
#ifndef LCD_TEXT_H
#define LCD_TEXT_H

//============================================================================//

#define LCD_DDR  DDRD
#define LCD_PORT PORTD

/* Pin-Belegung:
 * P0 - RS
 * P1 - R/W
 * P2 - E1
 * P3 - E2
 * P4 - D4
 * P5 - D5
 * P6 - D6
 * P7 - D7
 */ 
#define LCD_RS 0x01
#define LCD_RW 0x02
#define LCD_E1 0x04
#define LCD_E2 0x08

// Wenn die frequenz noch nicht gesetzt wurde hier setzten.
#ifndef F_CPU
	#define F_CPU 8000000UL
#endif

//============================================================================//

#include <inttypes.h>

#include <util/delay.h>
#include <avr/io.h>

//============================================================================//

void lcd_write(uint8_t data, uint8_t rs, uint8_t enable);

void lcd_control(uint8_t data, uint8_t enable);
void lcd_position(uint8_t zeile, uint8_t spalte, uint8_t enable);
void lcd_data(uint8_t data, uint8_t enable);
void lcd_clear(void);
void lcd_init(void);
void lcd_text(const char *text, uint8_t length, uint8_t enable);

//============================================================================//

void lcd_write(uint8_t data, uint8_t rs, uint8_t enable)
{

}

void lcd_control(uint8_t data, uint8_t enable)
{
	uint8_t port = 0x00;			//'Port' wird als ganzahlige Variable eingefhrt		

	port = (data & 0xF0);   			//Ausmaskieren der unteren 4 Bit

	LCD_PORT = port;              		//Ausgabe der Daten auf 'LCD_PORT' und RS = '0'
	LCD_PORT = port | enable;       		//Enable E 	= 	'1' setzen

	asm("nop");                   		//Warteschleife von min. 220 ns
	asm("nop");
	asm("nop");	

	LCD_PORT = port;              		//Enable E 	= 	'0' setzen

	port = (data & 0x0F) << 4;	   	//Ausmaskieren der oberen 4 Bit und verschieben auf Pin4 - Pin 7

	LCD_PORT = port;     							         	
	LCD_PORT = port | enable;       		//Enable E 	= 	'1' setzen

	asm("nop");                   		//Warteschleife von min. 220 ns
	asm("nop");
	asm("nop");

	LCD_PORT = port;              		//Enable E 	= 	'0' setzen

	_delay_us(50);						//Warteschleife von min. 42 us
}

//============================================================================//

void lcd_position(uint8_t zeile, uint8_t spalte, uint8_t enable)
{
	lcd_control(0x80 + (spalte - 1) + 0x40*(zeile - 1), enable);
}

//============================================================================//

void lcd_data(uint8_t data, uint8_t enable)
{
	uint8_t port = 0x00;			//'Port' wird als ganzahlige Variable eingefhrt		

	port = (data & 0xF0);   		//Ausmaskieren der unteren 4 Bit
	port = port | LCD_RS;			//RS = '1'

	LCD_PORT = port;              	//Ausgabe der Daten auf 'LCD_PORT' und RS = '0'
	LCD_PORT = port | enable;       	//Enable E 	= 	'1' setzen

	asm("nop");                   		//Warteschleife von min. 220 ns
	asm("nop");
	asm("nop");	

	LCD_PORT = port;              	//Enable E 	= 	'0' setzen

	port = (data & 0x0F) << 4;	//Ausmaskieren der oberen 4 Bit und verschieben auf 
	
	port = port | LCD_RS;			//RS = '1'
											
	LCD_PORT = port;     							         	
	LCD_PORT = port | enable;       	//Enable E 	= 	'1' setzen

	asm("nop");                   		//Warteschleife von min. 220 ns
	asm("nop");	
	asm("nop");

	LCD_PORT = port;              	//Enable E 	= 	'0' setzen

	_delay_us(50);						//Warteschleife von min. 42 us
}

//============================================================================//

void lcd_clear(void)
{
	lcd_control(0x01, LCD_E1);         		//Display wird 'geleert' 
	lcd_control(0x01, LCD_E2);      		//Display wird 'geleert'

	_delay_ms (2);						//Warteschleife von min. 1,64 ms
}

//============================================================================//

void lcd_init(void)
{
	_delay_ms(17);              			//Display muss min. 15 ms warten, nachdem die Betriebsspannung angelegt wurde
	lcd_control(0x28, LCD_E1);           		//System set:	N = 1 - vierzeiliges Display, 4 - Bit - Modus

	_delay_ms(5);                			//Warteschleife von min. 4,1 ms 
	lcd_control(0x28, LCD_E1);                	

	_delay_us(120);                		//Warteschleife von min. 100 us
	lcd_control(0x28, LCD_E1);                	

	_delay_us(100);						//Warteschleife von etwa 100 us - Busy flag 

	lcd_control(0x0C, LCD_E1);          		//Display wird eingeschaltet, Cursor wird ausgeschaltet

	lcd_control(0x06, LCD_E1);        		//Schreibrichtung wird festgelegt 

	lcd_control(0x02, LCD_E1);           		//Platzierung des Cursors an Position '0'
	_delay_ms (2);						//Warteschleife von min. 1,64 ms

							

	_delay_ms(17);              			//Display muss min. 15 ms warten, nachdem die Betriebsspannung angelegt wurde
	lcd_control(0x28, LCD_E2);          		//System set:	N = 1 - vierzeiliges Display, 4 - Bit - Modus

	_delay_ms(5);                			//Warteschleife von min. 4,1 ms 
	lcd_control(0x28, LCD_E2);                	

	_delay_us(120);                		//Warteschleife von min. 100 us
	lcd_control(0x28, LCD_E2);                	

	_delay_us(100);						//Warteschleife von etwa 100 us - Busy flag 

	lcd_control(0x0C, LCD_E2);           		//Display wird eingeschaltet, Cursor wird ausgeschaltet

	lcd_clear();                  		//Display wird gelscht

	lcd_control(0x06, LCD_E2);          		//Schreibrichtung wird festgelegt 

	lcd_control(0x02, LCD_E2);        		//Platzierung des Cursors an Position '0'
	_delay_ms (2);						//Warteschleife von min. 1,64 ms
}

//============================================================================//

void lcd_text(const char *text, uint8_t length, uint8_t enable)
{
	uint8_t i;
	
	for(i = 0; i < length; i++)
	{
		lcd_data(text[i], enable);
	}
}

//============================================================================//

#endif // LCD_TEXT_H



